#!/usr/bin/env python
# -*- coding: utf-8 -*-
import responses
import uuid
from pydrill.client.result import Profiles, Result


def test_profiles(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    profiles = pydrill_instance.profiles()
    assert type(profiles) == Profiles
    assert profiles.response.status_code == 200
    assert type(profiles.running_queries) == type([])
    assert type(profiles.finished_queries) == type([])


@responses.activate
def test_profile(pydrill_instance, pydrill_url):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    query_id = uuid.uuid4()

    responses.add(**{
        'method': responses.GET,
        'url': "{0}/{1}".format(pydrill_url, 'profiles/{0}.json'.format(query_id)),
        'content_type': 'application/json',
        'status': 200,
    })

    result = pydrill_instance.profile(query_id=query_id)
    assert type(result) == Result
    assert result.response.status_code == 200


@responses.activate
def test_profile_cancel(pydrill_instance, pydrill_url):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    query_id = uuid.uuid4()

    responses.add(**{
        'method': responses.GET,
        'url': "{0}/{1}".format(pydrill_url, 'profiles/cancel/{0}'.format(query_id)),
        'content_type': 'application/json',
        'status': 200,
    })

    result = pydrill_instance.profile_cancel(query_id=query_id)
    assert type(result) == Result
    assert result.response.status_code == 200



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pytest
import responses

from pydrill.client.result import Result


def test_storage(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    result = pydrill_instance.storage()
    assert type(result) == Result
    assert result.response.status_code == 200


@pytest.mark.parametrize('name', [
    'mongo', 'hive', 'cp', 'json', 'dfs', 's3', 'kudu', 'hbase'
])
def test_storage_detail(pydrill_instance, name):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    result = pydrill_instance.storage_detail(name=name)
    assert type(result) == Result
    assert result.response.status_code == 200


@pytest.mark.parametrize('name', [
    'mongo', 'hive', 'cp', 'json', 'dfs', 's3', 'kudu', 'hbase'
])
@pytest.mark.parametrize('value', [
    False, True
])
@responses.activate
def test_storage_enable(pydrill_instance, pydrill_url, name, value):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    responses.add(**{
        'method': responses.GET,
        'url': '{0}/{1}'.format(pydrill_url, 'storage/{0}/enable/{1}'.format(name, 'true' if value else 'false')),
        'content_type': 'application/json',
        'status': 200,
    })
    result = pydrill_instance.storage_enable(name=name, value=value)
    assert type(result) == Result
    assert result.response.status_code == 200


@pytest.mark.parametrize('name', [
    'mongo', 'hive', 'cp', 'dfs', 's3', 'kudu', 'hbase'
])
def test_storage_update(pydrill_instance, name):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    result = pydrill_instance.storage_detail(name=name)
    result = pydrill_instance.storage_update(name=name, config=result.data)
    assert type(result) == Result
    assert result.response.status_code == 200


@pytest.mark.parametrize('name', [
    'mongo', 'hive', 'cp', 'json', 'dfs', 's3', 'kudu', 'hbase'
])
@responses.activate
def test_storage_delete(pydrill_instance, pydrill_url, name):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    responses.add(**{
        'method': responses.DELETE,
        'url': '{0}/{1}'.format(pydrill_url, 'storage/{0}.json'.format(name)),
        'content_type': 'application/json',
        'status': 200,
    })
    result = pydrill_instance.storage_delete(name=name)
    assert type(result) == Result
    assert result.response.status_code == 200



import pytest
from pydrill.client import PyDrill


@pytest.fixture(scope='function', autouse=True)
def pydrill_instance():
    drill = PyDrill()
    return drill


@pytest.fixture()
def pydrill_url(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    return pydrill_instance.transport.connection.base_url



# -*- coding: utf-8 -*-


from pydrill.exceptions import ConnectionError, ConnectionTimeout, TransportError
from pydrill.serializer import Deserializer, JSONSerializer


class Transport(object):
    """
    Encapsulation of transport-related to logic. Handles instantiation of the
    individual connection.
    Main interface is the `perform_request` method.
    """

    def __init__(self, host, port, connection_class, serializers=None, default_mimetype='application/json',
                 max_retries=3, retry_on_status=(503, 504,), serializer=JSONSerializer(), deserializer=Deserializer(),
                 retry_on_timeout=False, send_get_body_as='GET', **kwargs):
        self.deserializer = deserializer
        self.port = port
        self.host = host
        self.connection = connection_class(host, port, **kwargs)
        self.retry_on_status = retry_on_status
        self.serializers = serializers
        self.retry_on_timeout = retry_on_timeout
        self.default_mimetype = default_mimetype
        self.max_retries = max_retries
        self.send_get_body_as = send_get_body_as
        self.serializer = serializer

    def perform_request(self, method, url, params=None, body=None):
        """
        Perform the actual request.
        Retrieve a connection.
        Pass all the information to it's perform_request method and return the data.

        :arg method: HTTP method to use
        :arg url: absolute url (without host) to target
        :arg params: dictionary of query parameters, will be handed over to the
            underlying :class:`~pydrill.Connection` class for serialization
        :arg body: body of the request, will be serializes using serializer and
            passed to the connection
        """
        if body is not None:
            body = self.serializer.dumps(body)

            # some clients or environments don't support sending GET with body
            if method in ('HEAD', 'GET') and self.send_get_body_as != 'GET':
                # send it as post instead
                if self.send_get_body_as == 'POST':
                    method = 'POST'

                # or as source parameter
                elif self.send_get_body_as == 'source':
                    if params is None:
                        params = {}
                    params['source'] = body
                    body = None

        if body is not None:
            try:
                body = body.encode('utf-8')
            except (UnicodeDecodeError, AttributeError):
                # bytes/str - no need to re-encode
                pass

        ignore = ()
        timeout = None
        if params:
            timeout = params.pop('request_timeout', None)
            ignore = params.pop('ignore', ())
            if isinstance(ignore, int):
                ignore = (ignore,)

        for attempt in range(self.max_retries + 1):
            connection = self.get_connection()

            try:
                response, data, duration = connection.perform_request(method, url, params, body, ignore=ignore,
                                                                      timeout=timeout)
            except TransportError as e:
                retry = False
                if isinstance(e, ConnectionTimeout):
                    retry = self.retry_on_timeout
                elif isinstance(e, ConnectionError):
                    retry = True
                elif e.status_code in self.retry_on_status:
                    retry = True

                if retry:
                    if attempt == self.max_retries:
                        raise
                else:
                    raise
            else:
                if data:
                    data = self.deserializer.loads(data, mimetype=response.headers.get('Content-Type'))
                else:
                    data = {}
                return response, data, duration

    def get_connection(self):
        return self.connection



import sys

version = '0.0.2'

VERSION = tuple(map(int, version.split('.')))
__version__ = VERSION
__versionstr__ = version


if (2, 7) <= sys.version_info < (3, 2):
    # <https://docs.python.org/2/howto/logging.html#configuring-logging-for-a-library>
    import logging
    logger = logging.getLogger('pydrill')
    logger.addHandler(logging.NullHandler())



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from pydrill.client.result import Result


def test_options(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    threads = pydrill_instance.options()
    assert type(threads) == Result
    assert threads.response.status_code == 200



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import os
import responses
from pydrill.exceptions import TransportError


def test_transport_host(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    assert pydrill_instance.transport.host == os.environ.get('PYDRILL_HOST', 'localhost')


def test_transport_port(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    assert pydrill_instance.transport.port == os.environ.get('PYDRILL_PORT', 8047)


def test_is_active(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    assert pydrill_instance.is_active() == True


@responses.activate
def test_is_not_active_404(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    responses.add(**{
        'method': responses.HEAD,
        'url': 'http://localhost:8047/',
        'content_type': 'application/json',
        'status': 404,
    })
    assert pydrill_instance.is_active() == False


@responses.activate
def test_is_not_active_500(pydrill_instance, pydrill_url):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    responses.add(**{
        'method': responses.HEAD,
        'url': pydrill_url,
        'content_type': 'application/json',
        'status': 500,
    })
    assert pydrill_instance.is_active() == False


@responses.activate
def test_is_not_active_201(pydrill_instance, pydrill_url):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    responses.add(**{
        'method': responses.HEAD,
        'url': pydrill_url,
        'content_type': 'application/json',
        'status': 201,
    })
    assert pydrill_instance.is_active() == False


def test_is_not_active_timeout(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    try:
        pydrill_instance.perform_request('HEAD', '/', params={'request_timeout': 0})
    except TransportError as e:
        assert e.status_code == e.args[0]
        assert e.error == e.args[1]
        assert e.info == e.args[2]
        assert str(e)
    else:
        assert False

# TODO: create more tests checking other params.



#!/usr/bin/env python
# -*- coding: utf-8 -*-
import pytest
from pydrill.exceptions import QueryError, ImproperlyConfigured


def test_select_employee(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    sql = "SELECT * FROM cp.`employee.json` ORDER BY salary DESC LIMIT 1"
    expected_result = {'columns': ['birth_date', 'department_id', 'education_level', 'employee_id', 'first_name',
                                   'full_name', 'gender', 'hire_date', 'last_name', 'management_role', 'marital_status',
                                   'position_id', 'position_title', 'salary', 'store_id', 'supervisor_id'],
                       'rows': [{'last_name': 'Nowmer', 'marital_status': 'S', 'management_role': 'Senior Management',
                                 'store_id': '0', 'position_id': '1', 'birth_date': '1961-08-26',
                                 'education_level': 'Graduate Degree', 'gender': 'F', 'supervisor_id': '0',
                                 'salary': '80000.0', 'department_id': '1', 'position_title': 'President',
                                 'full_name': 'Sheri Nowmer', 'hire_date': '1994-12-01 00:00:00.0',
                                 'first_name': 'Sheri', 'employee_id': '1'}]}

    result = pydrill_instance.query(sql=sql)

    assert result.response.status_code == 200
    assert result.data == expected_result


def test_select_iterator(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    sql = "SELECT * FROM cp.`employee.json` ORDER BY salary DESC LIMIT 1"

    for row in pydrill_instance.query(sql=sql):
        assert type(row) is dict


def test_select_pandas(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    sql = "SELECT * FROM cp.`employee.json` ORDER BY salary DESC LIMIT 1"

    with pytest.raises(ImproperlyConfigured):
        df = pydrill_instance.query(sql=sql).to_dataframe()


def test_select_without_sql(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    sql = ""
    try:
        result = pydrill_instance.query(sql=sql)
    except QueryError as e:
        assert e


def test_plan_for_select_employee(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    sql = "SELECT * FROM cp.`employee.json` ORDER BY salary DESC LIMIT 1"
    result = pydrill_instance.plan(sql=sql)
    assert result.response.status_code == 200


# TODO: create more tests with other queries.



# -*- coding: utf-8 -*-

import os
from pydrill.transport import Transport
from pydrill.client.result import ResultQuery, Result, Stats, Profiles
from pydrill.connection.requests_conn import RequestsHttpConnection
from pydrill.exceptions import QueryError, ConnectionError, TransportError


class PyDrill(object):
    """
    >>> drill = PyDrill(host='localhost', port=8047)
    >>> drill.is_active()
    True
    """
    # TODO: create better docs.

    def __init__(self, host=os.environ.get('PYDRILL_HOST', 'localhost'), port=os.environ.get('PYDRILL_PORT', 8047),
                 trasport_class=Transport, connection_class=RequestsHttpConnection, **kwargs):

        self.transport = trasport_class(host, port, connection_class=connection_class, **kwargs)

    def perform_request(self, method, url, params=None, body=None):
        return self.transport.perform_request(method, url, params, body)

    def is_active(self, timeout=2):
        """
        :param timeout: int
        :return: boolean
        """
        try:
            result = Result(*self.perform_request('HEAD', '/', params={'request_timeout': timeout}))
        except ConnectionError:
            return False
        except TransportError:
            return False

        if result.response.status_code == 200:
            return True

        return False

    def query(self, sql, timeout=10):
        """
        Submit a query and return results.

        :param sql: string
        :param timeout: int
        :return: pydrill.client.ResultQuery
        """
        if not sql:
            raise QueryError('No query passed to drill.')

        result = ResultQuery(*self.perform_request(**{
            'method': 'POST',
            'url': '/query.json',
            'body': {
                "queryType": "SQL",
                "query": sql
            },
            'params': {
                'request_timeout': timeout
            }
        }))

        return result

    def plan(self, sql, timeout=10):
        """
        :param sql: string
        :param timeout: int
        :return: pydrill.client.ResultQuery
        """
        sql = 'explain plan for ' + sql
        return self.query(sql, timeout)

    def stats(self, timeout=10):
        """
        Get Drillbit information, such as ports numbers.

        :param timeout: int
        :return: pydrill.client.Stats
        """
        result = Stats(*self.perform_request(**{
            'method': 'GET',
            'url': '/stats.json',
            'params': {
                'request_timeout': timeout
            }
        }))
        return result

    def metrics(self, timeout=10):
        """
        Get the current memory metrics.

        :param timeout: int
        :return: pydrill.client.Result
        """
        result = Result(*self.perform_request(**{
            'method': 'GET',
            'url': '/status/metrics',
            'params': {
                'request_timeout': timeout
            }
        }))
        return result

    def threads(self, timeout=10):
        """
        Get the status of threads.

        :param timeout: int
        :return: pydrill.client.Result
        """
        result = Result(*self.perform_request(**{
            'method': 'GET',
            'url': '/status/threads',
            'params': {
                'request_timeout': timeout
            }
        }))
        return result

    def options(self, timeout=10):
        """
        List the name, default, and data type of the system and session options.

        :param timeout: int
        :return: pydrill.client.Result
        """
        result = Result(*self.perform_request(**{
            'method': 'GET',
            'url': '/options.json',
            'params': {
                'request_timeout': timeout
            }
        }))
        return result

    def storage(self, timeout=10):
        """
        Get the list of storage plugin names and configurations.

        :param timeout: int
        :return: pydrill.client.Result
        """
        result = Result(*self.perform_request(**{
            'method': 'GET',
            'url': '/storage.json',
            'params': {
                'request_timeout': timeout
            }
        }))
        return result

    def storage_detail(self, name, timeout=10):
        """
        Get the definition of the named storage plugin.

        :param name: The assigned name in the storage plugin definition.
        :param timeout: int
        :return: pydrill.client.Result
        """
        result = Result(*self.perform_request(**{
            'method': 'GET',
            'url': '/storage/{0}.json'.format(name),
            'params': {
                'request_timeout': timeout
            }
        }))
        return result

    def storage_enable(self, name, value=True, timeout=10):
        """
        Enable or disable the named storage plugin.

        :param name: The assigned name in the storage plugin definition.
        :param value: Either True (to enable) or False (to disable).
        :param timeout: int
        :return: pydrill.client.Result
        """
        value = 'true' if value else 'false'
        result = Result(*self.perform_request(**{
            'method': 'GET',
            'url': '/storage/{0}/enable/{1}'.format(name, value),
            'params': {
                'request_timeout': timeout
            }
        }))
        return result

    def storage_update(self, name, config, timeout=10):
        """
        Create or update a storage plugin configuration.

        :param name: The name of the storage plugin configuration to create or update.
        :param config: Overwrites the existing configuration if there is any, and therefore, must include all
        required attributes and definitions.
        :param timeout: int
        :return: pydrill.client.Result
        """
        result = Result(*self.perform_request(**{
            'method': 'POST',
            'url': '/storage/{0}.json'.format(name),
            'body': config,
            'params': {
                'request_timeout': timeout
            }
        }))
        return result

    def storage_delete(self, name, timeout=10):
        """
        Delete a storage plugin configuration.

        :param name: The name of the storage plugin configuration to delete.
        :param timeout: int
        :return: pydrill.client.Result
        """
        result = Result(*self.perform_request(**{
            'method': 'DELETE',
            'url': '/storage/{0}.json'.format(name),
            'params': {
                'request_timeout': timeout
            }
        }))
        return result

    def profiles(self, timeout=10):
        """
        Get the profiles of running and completed queries.

        :param timeout: int
        :return: pydrill.client.Result
        """
        result = Profiles(*self.perform_request(**{
            'method': 'GET',
            'url': '/profiles.json',
            'params': {
                'request_timeout': timeout
            }
        }))
        return result

    def profile(self, query_id, timeout=10):
        """
        Get the profile of the query that has the given queryid.

        :param query_id: The UUID of the query in standard UUID format that Drill assigns to each query.
        :param timeout: int
        :return: pydrill.client.Result
        """
        result = Result(*self.perform_request(**{
            'method': 'GET',
            'url': '/profiles/{0}.json'.format(query_id),
            'params': {
                'request_timeout': timeout
            }
        }))
        return result

    def profile_cancel(self, query_id, timeout=10):
        """
        Cancel the query that has the given queryid.

        :param query_id: The UUID of the query in standard UUID format that Drill assigns to each query.
        :param timeout: int
        :return: pydrill.client.Result
        """
        result = Result(*self.perform_request(**{
            'method': 'GET',
            'url': '/profiles/cancel/{0}'.format(query_id),
            'params': {
                'request_timeout': timeout
            }
        }))
        return result



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from pydrill.client.result import Result


def test_threads(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    threads = pydrill_instance.threads()
    assert type(threads) == Result
    assert threads.response.status_code == 200
    assert type(threads.data) == type(u"")



# -*- coding: utf-8 -*-
from pydrill.exceptions import ImproperlyConfigured

try:
    import pandas as pd

    PANDAS_AVAILABLE = True
except ImportError:
    PANDAS_AVAILABLE = False


class Result(object):
    def __init__(self, response, data, duration, *args, **kwargs):
        self.response = response
        self.duration = duration
        self.data = data


class ResultQuery(Result):
    """
    Class responsible for maintaining a information returned from Drill.

    It is iterable.
    """
    def __init__(self, response, data, duration, *args, **kwargs):
        super(ResultQuery, self).__init__(response, data, duration, *args, **kwargs)
        self.rows = data.get('rows', [])
        self.columns = data.get('columns', [])

    def __iter__(self):
        for row in self.rows:
            yield row

    def to_dataframe(self):
        if not PANDAS_AVAILABLE:
            raise ImproperlyConfigured("Please install pandas to use ResultQuery.to_dataframe().")
        return pd.DataFrame.from_dict(self.rows)


class Drillbit(object):
    def __init__(self, id, address, status, *args, **kwargs):
        self.id = id
        self.address = address
        self.status = status


class Stats(Result):
    def __init__(self, response, data, duration, *args, **kwargs):
        super(Stats, self).__init__(response, data, duration, *args, **kwargs)

        self.drillbits = []

        for metric in data:
            value, name = metric['value'], metric['name']

            if name == 'Number of Drill Bits':
                self.drillbits_number = value
            elif name.startswith('Bit #'):
                address, status = value.split()
                self.drillbits.append(Drillbit(id=name.split('#')[-1], address=address, status=status))
            elif name == 'Data Port Address':
                self.data_port_address = value
            elif name == 'User Port Address':
                self.user_port_address = value
            elif name == 'Control Port Address':
                self.control_port_address = value
            elif name == 'Maximum Direct Memory':
                self.max_direct_memory = value


class Profiles(Result):
    def __init__(self, response, data, duration, *args, **kwargs):
        super(Profiles, self).__init__(response, data, duration, *args, **kwargs)
        self.running_queries = data.get('runningQueries')
        self.finished_queries = data.get('finishedQueries')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from pydrill.client.result import Stats, Result, Drillbit


def test_stats(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    stats = pydrill_instance.stats()
    assert type(stats) == Stats
    assert stats.response.status_code == 200
    assert stats.max_direct_memory

    assert stats.control_port_address
    assert stats.user_port_address
    assert stats.data_port_address

    assert stats.drillbits_number
    assert type(stats.drillbits) == type([])
    assert type(stats.drillbits[0]) == Drillbit


def test_metrics(pydrill_instance):
    """
    :type pydrill_instance: pydrill.client.PyDrill
    """
    result = pydrill_instance.metrics()
    assert type(result) == Result
    assert result.response.status_code == 200
    assert 'gauges' in result.data.keys()



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import find_packages
try:
    from setuptools import setup
except ImportError:
    from distutils.core import setup


with open('README.rst') as readme_file:
    readme = readme_file.read()

with open('HISTORY.rst') as history_file:
    history = history_file.read()

requirements = [
    'requests',
]

test_requirements = open('requriements_testing.txt', 'r').readlines()

setup(
    name='pydrill',
    version='0.0.2',
    description="Python Driver for Apache Drill.",
    long_description=readme + '\n\n' + history,
    author="Wojciech Nowak",
    author_email='mail@pythonic.ninja',
    url='https://github.com/PythonicNinja/pydrill',
    packages=find_packages(
        where='.',
        exclude=('test_*', )
    ),
    include_package_data=True,
    install_requires=requirements,
    license="MIT",
    zip_safe=False,
    keywords='pydrill',
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Natural Language :: English',
        "Programming Language :: Python :: 2",
        'Programming Language :: Python :: 2.6',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
        'Programming Language :: Python :: 3.5',
    ],
    test_suite='tests',
    tests_require=test_requirements
)
